package com.zfire.jiasm.syncdata.download.order;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.zfire.jiasm.syncdata.base.BaseService;
import com.zfire.jiasm.syncdata.constant.GDOrderStatusEnum;
import com.zfire.jiasm.syncdata.constant.OrderTypeEnum;
import com.zfire.jiasm.syncdata.dao.mapper.*;
import com.zfire.jiasm.syncdata.parameter.OrderPar;
import com.zfire.jiasm.syncdata.parameter.ReAppointment;
import com.zfire.jiasm.syncdata.plus.entity.*;
import com.zfire.jiasm.syncdata.plus.service.*;
import com.zfire.jiasm.syncdata.response.WebsitSH;
import com.zfire.jiasm.syncdata.service.OrderService;
import com.zfire.jiasm.syncdata.service.PublicService;
import lombok.extern.slf4j.Slf4j;
import org.jetbrains.annotations.NotNull;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 格力品牌维修工单下载后落本地数据表
 *    落业务表 - 修改下载主表处理结果
 */
@Slf4j
@Component
public class RepairBusiness {

    @Resource
    RepairBusiness repairBusiness;
    @Resource
    ItfTblAssignDownloadService itfTblAssignDownloadService;
    @Resource
    RepairDownloadMapper repairDownloadMapper;
    @Resource
    BaseService baseService;
    @Resource
    OrderBaseService orderBaseService;
    @Resource
    OrderService orderService;
    @Resource
    OrderApplyDispatchService orderApplyDispatchService;
    @Resource
    AssignDownloadMapper assignDownloadMapper;
    @Resource
    ItfTblAssignMxDownloadService itfTblAssignMxDownloadService;
    @Resource
    RepairMxDownloadMapper repairMxDownloadMapper;
    @Resource
    PublicService publicService;
    @Resource
    OrderProductService orderProductService;
    @Resource
    ItfTblAssignFkmxDownloadService itfTblAssignFkmxDownloadService;
    @Resource
    OrderRepairOperatingLogService orderRepairOperatingLogService;
    @Resource
    ItfTblAssignSatisfactionDownloadService itfTblAssignSatisfactionDownloadService;
    @Resource
    OrderAppraiseService orderAppraiseService;
    @Resource
    AppraiseMapper appraiseMapper;
    @Resource
    AssignSatisfactionDownloadMapper assignSatisfactionDownloadMapper;
    @Resource
    FaGreeSmsService faGreeSmsService;
    @Resource
    ItfTblAssignXzydDownloadService itfTblAssignXzydDownloadService;
    @Resource
    OrderInformationService orderInformationService;
    @Resource
    ItfTblAssignAppointmentDownloadService itfTblAssignAppointmentDownloadService;
    @Resource
    OrderRepairDetailService orderRepairDetailService;
    @Resource
    OrderRepairService orderRepairService;




    // 1：工单落入本地表
    @Scheduled(fixedDelay = 1 * 5 * 1000)
    public void inputOrder() {
        if (1 == 1) {
            return;
        }

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        List<ItfTblAssignDownload> repairRec = new ArrayList<>();
        List<Long> pgids = new ArrayList<>();
        List<OrderBase> orderBasesRecs = new ArrayList<>();
        List<String> orderBaselist = new ArrayList<>();

        // 提取品牌信息
        Brand brand = baseService.getBrand();
        if (brand == null) {
            return;
        }

        // 提取品牌商对应的大类信息
        Map<String, List<SysDictRef>> sysDictRefDLMap = publicService.getStringListMap(brand);
        // 提取品牌商对应的小类信息
        Map<String, List<SysDictRef>> sysDictRefXLMap = publicService.getListMap(brand);
        // 提取品牌商对应的系列信息
        Map<String, List<SysDictRef>> sysDictRefXiLieMap = publicService.getMap(brand);

        // 提取品牌商对应的需求大类信息
        Map<String, List<SysDictRef>> sysDictRefDLMapXq = publicService.getStringListMapXq(brand);
        // 提取品牌商对应的需求小类信息
        Map<String, List<SysDictRef>> sysDictRefXLMapXq = publicService.getListMapXq(brand);

        // 提取待落本地表的数据
        repairRec = itfTblAssignDownloadService.lambdaQuery()
                .eq(ItfTblAssignDownload::getSynStatus,0)
                .orderByAsc(ItfTblAssignDownload::getSynReqId)
                .list();

        if (repairRec.size()<1){
            log.info("暂无待处理的维修工单: \uF0B7{}",sdf.format(new Date()));
            return;
        }

        pgids = repairRec.stream().map(ItfTblAssignDownload::getPgid).collect(Collectors.toList());

        // 提取已有的订单数据
        orderBasesRecs = orderBaseService.lambdaQuery()
                .select(OrderBase::getPgid)
                .in(OrderBase::getPgid,pgids).list();
        orderBaselist = orderBasesRecs.stream().map(OrderBase::getPgid).collect(Collectors.toList());

        log.info("维修工单落本地表开始处理: \uF0B7{}",sdf.format(new Date()));

        // 按任务号download_task_no，按单来处理
        for (ItfTblAssignDownload item: repairRec){
            try{
                if (orderBaselist.contains(String.valueOf(item.getPgid()))){
                    // 修改
                    log.info("修改：");
                    repairBusiness.updateOrder(brand, sysDictRefDLMap, sysDictRefXLMap, sysDictRefXiLieMap, item,
                            sysDictRefDLMapXq, sysDictRefXLMapXq);
                } else {
                    // 新增 TL_SynDataFrom_tbl_assign_batch
                    log.info("新增：");
                    repairBusiness.addOrder(brand, item, sysDictRefDLMap, sysDictRefXLMap, sysDictRefXiLieMap,
                            sysDictRefDLMapXq, sysDictRefXLMapXq);
                    orderBaselist.add(String.valueOf(item.getPgid()));

                }
            }catch (Exception ex) {
                ex.printStackTrace();
                log.info("维修工单落本地表失败 销售单号-派工单号：【" + item.getXsdh() + "-"+ item.getPgid()+"】");
            }
        }

        log.info("维修工单落本地表处理完成: \uF0B7{}",sdf.format(new Date()));
    }

    @Transactional(rollbackFor = Exception.class)
    public void updateOrder(Brand brand, Map<String, List<SysDictRef>> sysDictRefDLMap,
                            Map<String, List<SysDictRef>> sysDictRefXLMap,
                            Map<String, List<SysDictRef>> sysDictRefXiLieMap,
                            ItfTblAssignDownload item,
                            Map<String, List<SysDictRef>> sysDictRefDLMapXq,
                            Map<String, List<SysDictRef>> sysDictRefXLMapXq) {
        List<ItfTblAssignXzydDownload> yds = itfTblAssignXzydDownloadService.lambdaQuery()
                .eq(ItfTblAssignXzydDownload::getDownloadTaskNo, item.getDownloadTaskNo()).list();

        // TL_SynDataFrom_tbl_assign_batch
        // TL_SynDataFrom_tbl_assign 主单的 syn_req_id
        // 处理维修工单主表数据
        modifyOrderBase(brand, sysDictRefDLMap, item);

        OrderBase order = orderBaseService.lambdaQuery()
                .eq(OrderBase::getPgid, item.getPgid())
                .eq(OrderBase::getOrderType, OrderTypeEnum.ORDER_TYPE_REPAIR.getCode()).one();

        // 工单师傅信息更新
        upOrderWorker(order);
        // 工单扩展信息更新
        getOrderInstall(item, order,sysDictRefDLMapXq, sysDictRefXLMapXq);
        // 维修单明细表更新 TL_SynDataFrom_tbl_assign_mx
        generateMxUpdate(brand, item, order, sysDictRefDLMap, sysDictRefXLMap, sysDictRefXiLieMap);
        // 处理维修单过程反馈(只新增不修改)  放入操作日志表 TL_SynDataFrom_tbl_assign_fkmx
        generateFkmxUpdate(brand, item, order);
        // 处理用户评价数据 TL_SynDataFrom_tbl_assign_satisfaction
        generateSstisfactionUpdate(brand, item, sysDictRefDLMap, order);
        // 阅读要求数据处理 TL_SynDataFrom_tbl_assign_xzyd
        generateXzydUpdate(brand, yds, order);
        // 用户预约改约数据处理 TL_SynDataFrom_tbl_assign_appointment
        generateAppointmentUpdate(item, order);
    }

    private void upOrderWorker(OrderBase order) {
        if (StringUtils.isEmpty(order.getWorkerName())){
            // 清理师傅表
            publicService.deleteWorker(order.getId());
        } else {
            // 提取现有的师傅信息
            List<OrderWorker> ows = publicService.getWorker(order.getWorkerId(), order.getId());
            // 看是工单师傅是否在内，如不在内说明重新指派了师傅
            Boolean flag = Boolean.FALSE;
            for (OrderWorker ow :ows){
                if (ow.getWorkerId().equals(order.getWorkerId())){
                    flag = Boolean.TRUE;
                }
            }
            if (!flag){
                // 清理师傅表，重新生新新师傅数据
                if (ows !=null && ows.size()>0){
                    publicService.deleteWorker(order.getId());
                }
                publicService.getOrderWorker(order);
            }
        }
    }

    private void modifyOrderBase(Brand brand, Map<String, List<SysDictRef>> sysDictRefDLMap, ItfTblAssignDownload item) {
        String orderStatus="";
        Integer cjDowFlag=0;
        // TL_SynDataFrom_tbl_assign_batch
        // TL_SynDataFrom_tbl_assign 主单的 syn_req_id
        // 提取基础表数据
        OrderBase oldOrder = orderBaseService.lambdaQuery()
                .eq(OrderBase::getPgid, item.getPgid())
                .eq(OrderBase::getOrderType, OrderTypeEnum.ORDER_TYPE_REPAIR.getCode()).one();
        orderStatus = oldOrder.getOrderStatus();
        cjDowFlag = oldOrder.getOutCjDownloadFlag();
        // 批量关闭的工单 同步下载无需再修改
        if (oldOrder.getOrderStatus().equals(GDOrderStatusEnum.REPAIR_STATUS_G13.getCode())){
            upItfTblAssignDownload(item.getSynReqId(),99,"last_modified_by = CLIENT__autoapp 本地已作废/关闭,忽略下载处理");
            return;
        }
        // 基础表数据同步 总部的修改时间必须晚于本地系统时间才可以修改
        if (item.getLastModifiedDate().compareTo(oldOrder.getUpdateTime())>0){
            upOrderBase(brand, item, oldOrder.getId());
            oldOrder = orderBaseService.lambdaQuery()
                    .eq(OrderBase::getPgid, item.getPgid())
                    .eq(OrderBase::getOrderType, OrderTypeEnum.ORDER_TYPE_REPAIR.getCode()).one();
        }
        // 网点信息更新
        updateWebsit(brand, sysDictRefDLMap, item, oldOrder);
        // 用户期望上门时间信息更新
        updateYhqwsmsj(item, oldOrder);
        // 状态信息更新
        upOrderStatus(brand, item, oldOrder);
        // 更新 工单机型
        upOrderProduct(oldOrder);
        // 更新 维修数据采集表
        upOrderRepairDetail(oldOrder);
        // 生成改派申请数据
        if (oldOrder.getOrderStatus().equals("B5")){
            generateDispatch(oldOrder);
        }
        // 修改工单改派申请表
        upOrderApplyDispatch(item, orderStatus, oldOrder);

        // 增加判断是否有未处理的待改派，如果广佛有待改派但总部工单不是待改派，直接更新改派标志
        // 无删除标识，暂不处理

        // 增加处理采集数据下载
        if ((oldOrder.getOrderStatus().equals("B41") || oldOrder.getOrderStatus().equals("B102")) && cjDowFlag ==0){
            List<OrderRepairDetail> ords = orderRepairDetailService.lambdaQuery()
                    .eq(OrderRepairDetail::getOrderBaseId,oldOrder.getId())
                    .ne(OrderRepairDetail::getStatus,0).list();
            // 已存在状态为"已采集"的记录暂时忽略下载
            if (!(ords !=null && ords.size()>0)){
                // 清除待采集数据
                delOrderRepairDetail(oldOrder);
                // 向表内写数据，另起任务下载数据 todo

                // 修改基础表已采集下载标识
                OrderBase up = new OrderBase();
                up.setId(oldOrder.getId());
                up.setOutCjDownloadFlag(1);
                orderBaseService.updateById(up);
            }
        }
        // 如果是家用的工单，且工单状态是待中心派工或待网点派工时，写入派嘉讯茂  暂停 todo

        // 修改同步状态
        upItfTblAssignDownloadStatus(item, oldOrder);
    }

    private void delOrderRepairDetail(OrderBase oldOrder) {
        QueryWrapper dWrapper = new QueryWrapper<OrderRepairDetail>()
                .eq("order_base_id", oldOrder.getId())
                .eq("status",0);
        orderRepairDetailService.remove(dWrapper);
    }

    private void upOrderApplyDispatch(ItfTblAssignDownload item, String orderStatus, OrderBase oldOrder) {
        // 原 本地维修单表状态为5 待审批  同步过来的工单状态不是待审批时
        // 修改工单改派申请 自动审核  新增时条件不成立的
        if (orderStatus.equals(GDOrderStatusEnum.REPAIR_STATUS_G5.getCode()) && !item.getStat().equals(5L)){
            OrderApplyDispatch oad = new OrderApplyDispatch();
            oad.setExamineId("system");
            oad.setExamineName("接口自动审核");
            oad.setExamineStatus("OK");
            oad.setExamineTime(new Date());
            QueryWrapper upWrapper = new QueryWrapper<OrderApplyDispatch>()
                    .eq("order_base_id", oldOrder.getId())
                    .eq("examine_status","WAIT");
            orderApplyDispatchService.update(oad,upWrapper);
        }
    }

    private void upOrderRepairDetail(OrderBase oldOrder) {
        OrderRepairDetail ord = new OrderRepairDetail();
        ord.setInstallWebsitId(oldOrder.getWebsitId());
        ord.setInstallWebsitName(oldOrder.getWebsitName());
        QueryWrapper upWrapper = new QueryWrapper<OrderRepairDetail>()
                .eq("order_base_id", oldOrder.getId());
        orderRepairDetailService.update(ord,upWrapper);
    }

    private void upOrderProduct(OrderBase oldOrder) {
        OrderProduct op = new OrderProduct();
        op.setServiceWebsit(oldOrder.getWebsitId());
        op.setServiceWebsitMobile(publicService.getSHWDPhone(oldOrder.getWebsitId()));
        QueryWrapper upWrapper = new QueryWrapper<OrderProduct>()
                .eq("order_base_id", oldOrder.getId());
        orderProductService.update(op,upWrapper);
    }

    private void upOrderStatus(Brand brand, ItfTblAssignDownload item, OrderBase oldOrder) {
        // 查看是否可更新
        Boolean allowUpdate = getaBoolean(item, oldOrder);
        if (allowUpdate){
            // 这里差 派工状态
            OrderPar orderPar = new OrderPar();
            orderPar.setSynReqId(item.getSynReqId());
            orderPar.setBrandId(brand.getBrandId());
            OrderBase order = repairDownloadMapper.getOrderBaseStatus(orderPar);
            order.setId(oldOrder.getId());
            orderBaseService.updateById(order);

            oldOrder = orderBaseService.lambdaQuery()
                    .eq(OrderBase::getPgid, item.getPgid())
                    .eq(OrderBase::getOrderType, OrderTypeEnum.ORDER_TYPE_REPAIR.getCode()).one();
        }
    }

    @NotNull
    private Boolean getaBoolean(ItfTblAssignDownload item, OrderBase oldOrder) {
        Boolean allowUpdate = Boolean.TRUE;

        if (oldOrder.getOrderStatus().equals(GDOrderStatusEnum.INSTALL_STATUS_G9.getCode())){
            allowUpdate = Boolean.FALSE;
        }
        if (oldOrder.getOrderStatus().equals(GDOrderStatusEnum.REPAIR_STATUS_G7.getCode()) &&
                (item.getStat().equals(4L) || item.getStat().equals(18L))){
            allowUpdate = Boolean.FALSE;
        }
        if (!oldOrder.getOrderStatus().equals(GDOrderStatusEnum.INSTALL_STATUS_G1_1.getCode()) &&
                !oldOrder.getOrderStatus().equals(GDOrderStatusEnum.INSTALL_STATUS_G2_0.getCode()) &&
                !oldOrder.getOrderStatus().equals(GDOrderStatusEnum.INSTALL_STATUS_G3.getCode()) &&
                !oldOrder.getOrderStatus().equals(GDOrderStatusEnum.INSTALL_STATUS_G4_0.getCode()) &&
                !oldOrder.getOrderStatus().equals(GDOrderStatusEnum.INSTALL_STATUS_G5.getCode()) &&
                !oldOrder.getOrderStatus().equals(GDOrderStatusEnum.INSTALL_STATUS_G6.getCode()) &&
                !oldOrder.getOrderStatus().equals(GDOrderStatusEnum.INSTALL_STATUS_G7.getCode()) &&
                item.getStat()<=4L){
            allowUpdate = Boolean.TRUE;
        }
        if (item.getStat().equals(2L) && (!oldOrder.getOrderStatus().equals(GDOrderStatusEnum.INSTALL_STATUS_G1_1.getCode()) &&
                !oldOrder.getOrderStatus().equals(GDOrderStatusEnum.INSTALL_STATUS_G2_0.getCode()) &&
                !oldOrder.getOrderStatus().equals(GDOrderStatusEnum.INSTALL_STATUS_G3.getCode()))){
            allowUpdate = Boolean.FALSE;
        }
        if (item.getStat().equals(2L) && (oldOrder.getOrderStatus().equals(GDOrderStatusEnum.INSTALL_STATUS_G5.getCode()) ||
                oldOrder.getOrderStatus().equals(GDOrderStatusEnum.REPAIR_STATUS_G11.getCode()))){
            allowUpdate = Boolean.TRUE;
        }
        return allowUpdate;
    }

    private void updateYhqwsmsj(ItfTblAssignDownload item, OrderBase oldOrder) {
        // 如何判断工单已预约 首次预约人信息不为空即为已预约
        if (StringUtils.isEmpty(oldOrder.getFirstAppointmentId())){
            OrderBase upOrder = new OrderBase();
            upOrder.setId(oldOrder.getId());
            upOrder.setExpectStartTime(item.getYhqwsmsj());
            upOrder.setExpectEndTime(item.getQwsmjssj());
            orderBaseService.updateById(upOrder);
        }
    }

    private void updateWebsit(Brand brand, Map<String, List<SysDictRef>> sysDictRefDLMap, ItfTblAssignDownload item, OrderBase oldOrder) {
        // 看网点更新的条件
        Boolean flag = getaBoolean(sysDictRefDLMap, item, oldOrder);
        // 更新网点信息
        if (flag){
            // 字段赋值
            getOrderBase(brand, item, oldOrder);
            orderBaseService.updateById(oldOrder);
        }
    }

    @NotNull
    private void getOrderBase(Brand brand, ItfTblAssignDownload item, OrderBase upOrder) {
        if (StringUtils.isNotEmpty(item.getXxlb())){
            upOrder.setInfoType(item.getXxlb());
        } else {
            upOrder.setInfoType("");
        }
        if (StringUtils.isNotEmpty(String.valueOf(item.getXxlbid()))){
            upOrder.setInfoTypeId(String.valueOf(item.getXxlbid()));
        } else {
            upOrder.setInfoTypeId("101");
        }
        // 维修网点
        String wdno ="";
        if (StringUtils.isEmpty(item.getWxwdno())){
            wdno = item.getXjwdno();
        } else {
            wdno = item.getWxwdno();
        }
        WebsitSH resultData = publicService.WebsitToSH(wdno, brand.getBrandId());
        upOrder.setWebsitId(resultData.getWdId());
        upOrder.setWebsitName(resultData.getWdmc());
        if (StringUtils.isNotEmpty(item.getXjwdno())){
            WebsitSH res = publicService.WebsitToSH(item.getXjwdno(), brand.getBrandId());
            upOrder.setReceWebsitId(res.getWdId());
            upOrder.setReceWebsitName(res.getWdmc());
        } else {
            upOrder.setReceWebsitId("");
            upOrder.setReceWebsitName("");
        }
    }

    @NotNull
    private Boolean getaBoolean(Map<String, List<SysDictRef>> sysDictRefDLMap, ItfTblAssignDownload item, OrderBase oldOrder) {
        Boolean flag = Boolean.FALSE;
        if ((oldOrder.getOrderStatus().equals(GDOrderStatusEnum.REPAIR_STATUS_G1.getCode()) ||
                oldOrder.getOrderStatus().equals(GDOrderStatusEnum.REPAIR_STATUS_G2.getCode()) ||
                oldOrder.getOrderStatus().equals(GDOrderStatusEnum.REPAIR_STATUS_G3.getCode()) ||
                oldOrder.getOrderStatus().equals(GDOrderStatusEnum.REPAIR_STATUS_G4.getCode()) ||
                oldOrder.getOrderStatus().equals(GDOrderStatusEnum.REPAIR_STATUS_G5.getCode()) ||
                oldOrder.getOrderStatus().equals(GDOrderStatusEnum.REPAIR_STATUS_G6.getCode())) ||
                (oldOrder.getOrderStatus().equals(GDOrderStatusEnum.REPAIR_STATUS_G7.getCode()) &&
                        !oldOrder.getDispatchStatus().equals("YCJ") &&
                        !oldOrder.getDispatchStatus().equals("YBWG"))){
            flag = Boolean.TRUE;
        }
        if ((!oldOrder.getOrderStatus().equals(GDOrderStatusEnum.REPAIR_STATUS_G1.getCode()) &&
                !oldOrder.getOrderStatus().equals(GDOrderStatusEnum.REPAIR_STATUS_G2.getCode()) &&
                !oldOrder.getOrderStatus().equals(GDOrderStatusEnum.REPAIR_STATUS_G3.getCode())) && item.getStat() == 2){
            flag = Boolean.FALSE;
        }
        // op.setMainId(sysDictRefDLMap.get(String.valueOf(rec.getSpid())).get(0).getDictCode());
        if (StringUtils.isNotEmpty(oldOrder.getMainId()) && StringUtils.isNotEmpty(String.valueOf(item.getSpid())) &&
                !oldOrder.getMainId().equals(sysDictRefDLMap.get(String.valueOf(item.getSpid())).get(0).getDictCode())){
            flag = Boolean.TRUE;
        }
        return flag;
    }

    private void upOrderBase(Brand brand, ItfTblAssignDownload item, String id) {
        // 获取维修工单主表要修改的数据
        OrderBase upOrder = getUpOrderBase(brand, item, id);
        // 省市区单独处理
        orderService.getPCAS(upOrder);
        orderBaseService.updateById(upOrder);
    }

    @NotNull
    private OrderBase getUpOrderBase(Brand brand, ItfTblAssignDownload item, String id) {
        OrderPar orderPar = new OrderPar();
        orderPar.setSynReqId(item.getSynReqId());
        orderPar.setBrandId(brand.getBrandId());
        OrderBase upOrder = repairDownloadMapper.getUpOrderBase(orderPar);
        upOrder.setId(id);
        upOrder.setOutUpdateSynReqId(item.getSynReqId());
        upOrder.setOutUpdateSynReqTime(new Date());
        // upOrder.setUpdateTime(new Date());
        return upOrder;
    }

    @Transactional(rollbackFor = Exception.class)
    public void addOrder(Brand brand, ItfTblAssignDownload item,
                          Map<String, List<SysDictRef>> sysDictRefDLMap,
                          Map<String, List<SysDictRef>> sysDictRefXLMap,
                          Map<String, List<SysDictRef>> sysDictRefXiLieMap,
                         Map<String, List<SysDictRef>> sysDictRefDLMapXq,
                         Map<String, List<SysDictRef>> sysDictRefXLMapXq) {

        List<ItfTblAssignXzydDownload> yds = itfTblAssignXzydDownloadService.lambdaQuery()
                .eq(ItfTblAssignXzydDownload::getDownloadTaskNo,item.getDownloadTaskNo()).list();

        // 新增 TL_SynDataFrom_tbl_assign_batch
        // TL_SynDataFrom_tbl_assign 主单的 syn_req_id
        // 处理维修工单主表数据
        OrderBase order = generateOrder(brand, item, yds);
        // 处理工单师傅
        publicService.getOrderWorker(order);
        // 获取工单扩展信息
        getOrderInstall(item, order, sysDictRefDLMapXq, sysDictRefXLMapXq);
        // 处理维修单明细表 TL_SynDataFrom_tbl_assign_mx
        generateMx(brand, item, order, sysDictRefDLMap, sysDictRefXLMap, sysDictRefXiLieMap);
        // 处理维修单过程反馈表 放入操作日志表 TL_SynDataFrom_tbl_assign_fkmx
        generateFkmx(brand, item, order);
        // 处理用户评价数据 TL_SynDataFrom_tbl_assign_satisfaction
        generateSstisfaction(brand, item, sysDictRefDLMap, order);
        // 阅读要求数据处理 TL_SynDataFrom_tbl_assign_xzyd
        generateXzyd(brand, yds, order);
        // 用户预约改约数据处理 TL_SynDataFrom_tbl_assign_appointment
        generateAppointment(item, order);
    }

    private void generateAppointment(ItfTblAssignDownload item, OrderBase order) {
        OrderBase up = new OrderBase();
        List<ItfTblAssignAppointmentDownload> us = new ArrayList<>();
        // 新增工单，直接修改主表预约时间最近一次的数据修改
        List<ItfTblAssignAppointmentDownload> appoint = itfTblAssignAppointmentDownloadService.lambdaQuery()
                .eq(ItfTblAssignAppointmentDownload::getDownloadTaskNo, item.getDownloadTaskNo())
                .orderByAsc(ItfTblAssignAppointmentDownload::getSynReqId).list();
        for (ItfTblAssignAppointmentDownload a:appoint){
            up.setId(order.getId());
            // 这里填写首次预约人信息同时也方便其他地方以此来判断该订单是否已预约
            up.setFirstAppointmentId(a.getCzren());
            up.setExpectStartTime(a.getKssj());
            up.setExpectEndTime(a.getJssj());

            ItfTblAssignAppointmentDownload u = new ItfTblAssignAppointmentDownload();
            u.setSynReqId(a.getSynReqId());
            u.setSynStatus(1);
            u.setSynTime(new Date());
            us.add(u);
        }
        if (appoint !=null && appoint.size()>0){
            orderBaseService.updateById(up);
            itfTblAssignAppointmentDownloadService.updateBatchById(us);
            us.clear();
        }
    }

    private void generateAppointmentUpdate(ItfTblAssignDownload item, OrderBase order) {
        OrderBase up = new OrderBase();
        List<ItfTblAssignAppointmentDownload> us = new ArrayList<>();
        // 新增工单，直接修改主表预约时间最近一次的数据修改
        List<ItfTblAssignAppointmentDownload> appoint = itfTblAssignAppointmentDownloadService.lambdaQuery()
                .eq(ItfTblAssignAppointmentDownload::getDownloadTaskNo, item.getDownloadTaskNo())
                .orderByAsc(ItfTblAssignAppointmentDownload::getSynReqId).list();
        for (ItfTblAssignAppointmentDownload a:appoint){
            up.setId(order.getId());
            // 这里填写首次预约人信息同时也方便其他地方以此来判断该订单是否已预约
            up.setFirstAppointmentId(a.getCzren());
            up.setExpectStartTime(a.getKssj());
            up.setExpectEndTime(a.getJssj());

            ItfTblAssignAppointmentDownload u = new ItfTblAssignAppointmentDownload();
            u.setSynReqId(a.getSynReqId());
            u.setSynStatus(1);
            u.setSynTime(new Date());
            us.add(u);
        }
        if (appoint !=null && appoint.size()>0){
            if (StringUtils.isEmpty(order.getFirstAppointmentId())){
                orderBaseService.updateById(up);
            }
            itfTblAssignAppointmentDownloadService.updateBatchById(us);
            us.clear();
        }
    }

    private void generateXzyd(Brand brand, List<ItfTblAssignXzydDownload> yds, OrderBase order) {
        List<OrderInformation> oilist = new ArrayList<>();
        List<ItfTblAssignXzydDownload> ups = new ArrayList<>();

        for (ItfTblAssignXzydDownload yd: yds){
            // 获取阅读要求数据
            getAddOrderInfor(brand, order, oilist, yd);

            ItfTblAssignXzydDownload up = new ItfTblAssignXzydDownload();
            up.setSynReqId(yd.getSynReqId());
            up.setSynStatus(1);
            up.setSynTime(new Date());
            ups.add(up);
        }
        if (oilist.size()>0){
            orderInformationService.saveBatch(oilist);
            oilist.clear();
        }
        if (ups.size()>0){
            itfTblAssignXzydDownloadService.updateBatchById(ups);
            ups.clear();
        }
    }

    private void generateXzydUpdate(Brand brand, List<ItfTblAssignXzydDownload> yds, OrderBase order) {
        List<OrderInformation> oilist = new ArrayList<>();
        List<OrderInformation> oilistUp = new ArrayList<>();
        List<ItfTblAssignXzydDownload> ups = new ArrayList<>();
        Map<String,List<OrderInformation>> oisMap = new HashMap<>();
        // 提取原有的数据
        List<OrderInformation> ois = orderInformationService.lambdaQuery()
                .eq(OrderInformation::getOrderBaseId,order.getId()).list();
        oisMap = ois.stream().collect(Collectors.groupingBy(OrderInformation::getId));


        for (ItfTblAssignXzydDownload yd: yds){
            // 存在修改，不存在新增
            if (oisMap.containsKey(String.valueOf(yd.getXzid()))){
                getUpOrderInfor(brand, oilistUp, yd);
            } else {
                getAddOrderInfor(brand, order, oilist, yd);
            }

            ItfTblAssignXzydDownload up = new ItfTblAssignXzydDownload();
            up.setSynReqId(yd.getSynReqId());
            up.setSynStatus(1);
            up.setSynTime(new Date());
            ups.add(up);
        }
        if (oilist.size()>0){
            orderInformationService.saveBatch(oilist);
            oilist.clear();
        }
        if (oilistUp.size()>0){
            orderInformationService.updateBatchById(oilistUp);
            oilistUp.clear();
        }
        if (ups.size()>0){
            itfTblAssignXzydDownloadService.updateBatchById(ups);
            ups.clear();
        }
    }

    private void getUpOrderInfor(Brand brand, List<OrderInformation> oilistUp, ItfTblAssignXzydDownload yd) {
        OrderInformation up = new OrderInformation();
        up.setId(String.valueOf(yd.getXzid()));
        up.setAskCategoryId(yd.getXzyqlb());
        up.setAskCategoryName(yd.getXzyq());
        up.setReadStatus(yd.getYdbz());
        up.setReadTime(yd.getYdsj());
        up.setReadName(yd.getYdrmc());
        WebsitSH websitSH = publicService.WebsitToSH(yd.getWdno(), brand.getBrandId());
        up.setWebsitId(websitSH.getWdId());
        up.setWebsitName(websitSH.getWdmc());
        oilistUp.add(up);
    }

    private void getAddOrderInfor(Brand brand, OrderBase order, List<OrderInformation> oilist, ItfTblAssignXzydDownload yd) {
        OrderInformation add = new OrderInformation();
        add.setId(String.valueOf(yd.getXzid()));
        add.setAskCategoryId(yd.getXzyqlb());
        add.setAskCategoryName(yd.getXzyq());
        add.setOrderBaseId(order.getId());
        add.setCreateBy(yd.getCreatedBy());
        add.setCreateTime(yd.getCreatedDate());
        add.setUpdateBy(yd.getLastModifiedBy());
        add.setUpdateTime(yd.getLastModifiedDate());
        add.setReadStatus(yd.getYdbz());
        add.setReadName(yd.getYdrmc());
        add.setReadTime(yd.getYdsj());
        WebsitSH websitSH = publicService.WebsitToSH(yd.getWdno(), brand.getBrandId());
        add.setWebsitId(websitSH.getWdId());
        add.setWebsitName(websitSH.getWdmc());
        oilist.add(add);
    }

    private void generateSstisfaction(Brand brand, ItfTblAssignDownload item, Map<String, List<SysDictRef>> sysDictRefDLMap, OrderBase order) {
        List<ItfTblAssignSatisfactionDownload> satis = itfTblAssignSatisfactionDownloadService.lambdaQuery()
                .eq(ItfTblAssignSatisfactionDownload::getDownloadTaskNo, item.getDownloadTaskNo())
                .orderByAsc(ItfTblAssignSatisfactionDownload::getHfsj)
                .list();

        for (ItfTblAssignSatisfactionDownload s:satis){
            // 查看是否已同步
            ReAppointment request = new ReAppointment();
            request.setId(s.getId());
            OrderAppraise old = appraiseMapper.getAppraise(request);
            if (old!=null){
                upSyncstatus(s.getSynReqId(), 9, "评价已同步，忽略更新");
                continue;
            }
            // 查看该工单有无较早前下载相同的评价
            List<ItfTblAssignSatisfactionDownload> rec = itfTblAssignSatisfactionDownloadService.lambdaQuery()
                    .lt(ItfTblAssignSatisfactionDownload::getSynReqId,s.getSynReqId())
                    .eq(ItfTblAssignSatisfactionDownload::getPgid,s.getPgid())
                    .eq(ItfTblAssignSatisfactionDownload::getId,s.getId())
                    .le(ItfTblAssignSatisfactionDownload::getLastModifiedDate,s.getLastModifiedDate())
                    .eq(ItfTblAssignSatisfactionDownload::getPjly,s.getPjly()).list();
            if (rec!=null && rec.size()>0){
                upSyncstatus(s.getSynReqId(), 9 ,"评价已同步，忽略更新");
                continue;
            }
            // 判断评价是否为中差评,下面处理

            Integer canInsert =1;
            // 检查fa_order_repair是否有一条sxlx = 0的首评记录
            if (s.getSxlx().equals(0) && s.getSynStatus().equals(0)){
                ReAppointment r = new ReAppointment();
                r.setId(String.valueOf(s.getSynReqId()));
                ItfTblAssignSatisfactionDownload sa = assignSatisfactionDownloadMapper.getSatisfaction(r);
                if (sa !=null){
                    canInsert = 0;
                }
            }
            // 只插入一条sxlx = 0的首评记录
            if (canInsert.equals(1)){
                // 增加评价记录
                WebsitSH websitSH = addappraise(brand, sysDictRefDLMap, s);
                // 增加差评短信
                addcpsms(order, s, websitSH);
            }

            // 已评价过 新增时基础表不再操作追评 因为工单基础表无追评数据可存
            if (order.getAppraiseTime() ==null){
                // 这里为首次评价
                upOrderBasePJ(order, s);
            }

            // 修改同步状态
            upSatisfactionSynstatus(s);
        }
    }

    private void generateSstisfactionUpdate(Brand brand, ItfTblAssignDownload item, Map<String, List<SysDictRef>> sysDictRefDLMap, OrderBase order) {
        List<ItfTblAssignSatisfactionDownload> satis = itfTblAssignSatisfactionDownloadService.lambdaQuery()
                .eq(ItfTblAssignSatisfactionDownload::getDownloadTaskNo, item.getDownloadTaskNo())
                .orderByAsc(ItfTblAssignSatisfactionDownload::getHfsj)
                .list();

        for (ItfTblAssignSatisfactionDownload s:satis){
            // 查看是否已同步
            ReAppointment request = new ReAppointment();
            request.setId(s.getId());
            OrderAppraise old = appraiseMapper.getAppraise(request);
            if (old!=null){
                upSyncstatus(s.getSynReqId(), 9, "评价已同步，忽略更新");
                continue;
            }
            // 查看该工单有无较早前下载相同的评价(同一工单，同一主键，同一内容，更早时间)
            List<ItfTblAssignSatisfactionDownload> rec = itfTblAssignSatisfactionDownloadService.lambdaQuery()
                    .lt(ItfTblAssignSatisfactionDownload::getSynReqId,s.getSynReqId())
                    .eq(ItfTblAssignSatisfactionDownload::getPgid,s.getPgid())
                    .eq(ItfTblAssignSatisfactionDownload::getId,s.getId())
                    .le(ItfTblAssignSatisfactionDownload::getLastModifiedDate,s.getLastModifiedDate())
                    .eq(ItfTblAssignSatisfactionDownload::getPjly,s.getPjly()).list();
            if (rec!=null && rec.size()>0){
                upSyncstatus(s.getSynReqId(), 9 ,"评价已同步，忽略更新");
                continue;
            }
            // 判断评价是否为中差评,下面处理

            Integer canInsert =1;
            // 检查fa_order_repair是否有一条sxlx = 0的首评记录
            if (s.getSxlx().equals(0) && s.getSynStatus().equals(0)){
                ReAppointment r = new ReAppointment();
                r.setId(String.valueOf(s.getSynReqId()));
                ItfTblAssignSatisfactionDownload sa = assignSatisfactionDownloadMapper.getSatisfaction(r);
                if (sa !=null){
                    canInsert = 0;
                }
            }
            // 只插入一条sxlx = 0的首评记录
            if (canInsert.equals(1)){
                // 增加评价记录
                WebsitSH websitSH = addappraise(brand, sysDictRefDLMap, s);
                // 增加差评短信
                addcpsms(order, s, websitSH);
            }

            // 未评价过工单基础表写入首次评价数据
            if (order.getAppraiseTime() ==null){
                upOrderBasePJ(order, s);
            }

            // 修改同步状态
            upSatisfactionSynstatus(s);
        }
    }

    private void addcpsms(OrderBase order, ItfTblAssignSatisfactionDownload s, WebsitSH websitSH) {
        if (s.getPjnr().equals("3") || s.getPjnr().indexOf("不满意")>0){
            if (StringUtils.isNotEmpty(s.getHfwdno()) && s.getHfwdno().equals("99999999")){
                FaGreeSms fs = new FaGreeSms();
                fs.setSynTaskNo(UUID.randomUUID().toString());
                fs.setWorkerOrderNo(order.getId());
                fs.setAzorwx(2);
                fs.setFjid(String.valueOf(s.getPgid()));
                fs.setFjwd(s.getHfwdno());
                Random random = new Random(System.currentTimeMillis());
                fs.setFsma(String.valueOf(random.nextInt(10000)));
                fs.setFsnr("您收到一张客户的投诉工单，请及时处理！工单号：" + String.valueOf(s.getPgid()) +
                        ",服务人员：" + order.getWorkerName());
                fs.setSjid(websitSH.getWdlxdh());
                fs.setYhlb(2);
                faGreeSmsService.save(fs);
            }
        }
    }

    @NotNull
    private WebsitSH addappraise(Brand brand, Map<String, List<SysDictRef>> sysDictRefDLMap, ItfTblAssignSatisfactionDownload s) {
        Boolean canAppeal = Boolean.TRUE;
        Integer appeal = 0;
        ReAppointment rq = new ReAppointment();

        if(StringUtils.isNotEmpty(s.getPjnr())){
            if (s.getPjnr().equals("1") || s.getPjnr().equals("满意")){
                canAppeal=Boolean.FALSE;
            }
        }
        // 检查上一条记录是否为中差评，如果是则新插入的不需要再申诉
        if (canAppeal){
            ReAppointment req = new ReAppointment();
            req.setId(String.valueOf(s.getPgid()));
            OrderAppraise id = appraiseMapper.getOldId(req);
            if (id !=null){
                appeal = 0;
            } else {
                appeal = 1;
            }
        }

        rq.setId(String.valueOf(s.getSynReqId()));
        OrderAppraise add = assignSatisfactionDownloadMapper.getOrderAppraise(rq);

        // rvisit_websit_id 单独处理
        WebsitSH websitSH = publicService.WebsitToSH(s.getHfwdno(), brand.getBrandId());
        add.setRvisitWebsitId(websitSH.getWdId());
        add.setRvisitWebsitName(websitSH.getWdmc());
        // splb 商品大类  单独处理
        add.setMainId(sysDictRefDLMap.get(String.valueOf(s.getSplb())).get(0).getDictCode());
        // can_appeal 是否可发起申诉   单独处理
        add.setCanAppeal(appeal);
        orderAppraiseService.save(add);
        return websitSH;
    }

    private void upOrderBasePJ(OrderBase order, ItfTblAssignSatisfactionDownload s) {
        OrderBase upOrder = new OrderBase();
        upOrder.setId(order.getId());
        upOrder.setAppraiseTime(s.getCreatedDate());
        // 原 评价:1=差评,2=中评,3=好评,0=未评价,4=剔除,5=其他
        if (StringUtils.isEmpty(s.getPjnr())){
            upOrder.setAppraiseStatus("N");
        }
        if (s.getPjnr().equals("1") || s.getPjnr().equals("满意")){
            upOrder.setAppraiseStatus("A");
        } else if (s.getPjnr().equals("2") || s.getPjnr().equals("一般")){
            upOrder.setAppraiseStatus("B");
        } else if (s.getPjnr().equals("3") || s.getPjnr().equals("不满意") ||
                s.getPjnr().indexOf("差")>0 || s.getPjnr().indexOf("不满")>0){
            upOrder.setAppraiseStatus("C");
        } else {
            upOrder.setAppraiseStatus("O");
        }
        orderBaseService.updateById(upOrder);
    }

    private void upSatisfactionSynstatus(ItfTblAssignSatisfactionDownload s) {
        ItfTblAssignSatisfactionDownload ups = new ItfTblAssignSatisfactionDownload();
        ups.setSynReqId(s.getSynReqId());
        ups.setSynStatus(1);
        ups.setSynTime(new Date());
        itfTblAssignSatisfactionDownloadService.updateById(ups);
    }

    private void upSyncstatus(Long synReqId, Integer synStatus ,
                              String msg) {
        ItfTblAssignSatisfactionDownload up = new ItfTblAssignSatisfactionDownload();
        up.setSynReqId(synReqId);
        up.setSynStatus(synStatus);
        up.setSynTime(new Date());
        up.setSynErrMsg(msg);
        itfTblAssignSatisfactionDownloadService.updateById(up);
    }

    private void generateFkmx(Brand brand, ItfTblAssignDownload item, OrderBase order) {
        List<ItfTblAssignFkmxDownload> fkmxRec = new ArrayList<>();
        List<OrderRepairOperatingLog> orols = new ArrayList<>();
        List<ItfTblAssignFkmxDownload> ups = new ArrayList<>();

        fkmxRec = itfTblAssignFkmxDownloadService.lambdaQuery()
                .eq(ItfTblAssignFkmxDownload::getDownloadTaskNo, item.getDownloadTaskNo())
                .eq(ItfTblAssignFkmxDownload::getSynStatus,0)
                .orderByAsc(ItfTblAssignFkmxDownload::getCreatedDate,ItfTblAssignFkmxDownload::getFkid)
                .list();
        for (ItfTblAssignFkmxDownload fkmx :fkmxRec){
            // 获取操作明细数据
            getoroldata(brand, order, orols, fkmx);
            ItfTblAssignFkmxDownload u = new ItfTblAssignFkmxDownload();
            u.setSynReqId(fkmx.getSynReqId());
            u.setSynStatus(1);
            u.setSynTime(new Date());
            ups.add(u);
        }
        if (orols.size()>0){
            orderRepairOperatingLogService.saveBatch(orols);
            orols.clear();
            itfTblAssignFkmxDownloadService.updateBatchById(ups);
            ups.clear();
        }
    }

    private void generateFkmxUpdate(Brand brand, ItfTblAssignDownload item, OrderBase order) {
        List<ItfTblAssignFkmxDownload> fkmxRec = new ArrayList<>();
        List<OrderRepairOperatingLog> orols = new ArrayList<>();
        List<ItfTblAssignFkmxDownload> ups = new ArrayList<>();
        Map<String, List<OrderRepairOperatingLog>> orolMap = new HashMap<>();
        
        // 提取已有的，有则更新，无则新增
        List<OrderRepairOperatingLog> olds = orderRepairOperatingLogService.lambdaQuery()
                .eq(OrderRepairOperatingLog::getOrderBaseId,order.getId()).list();
        orolMap = olds.stream().collect(Collectors.groupingBy(OrderRepairOperatingLog::getFkid));
        

        fkmxRec = itfTblAssignFkmxDownloadService.lambdaQuery()
                .eq(ItfTblAssignFkmxDownload::getDownloadTaskNo, item.getDownloadTaskNo())
                .eq(ItfTblAssignFkmxDownload::getSynStatus,0)
                .orderByAsc(ItfTblAssignFkmxDownload::getCreatedDate,ItfTblAssignFkmxDownload::getFkid)
                .list();
        for (ItfTblAssignFkmxDownload fkmx :fkmxRec){
            // 存在不做更新？过程是这样写的
            if (!orolMap.containsKey(fkmx.getFkid())){
                getoroldata(brand, order, orols, fkmx);
            }
            
            ItfTblAssignFkmxDownload u = new ItfTblAssignFkmxDownload();
            u.setSynReqId(fkmx.getSynReqId());
            u.setSynStatus(1);
            u.setSynTime(new Date());
            ups.add(u);
        }
        if (orols.size()>0){
            orderRepairOperatingLogService.saveBatch(orols);
            orols.clear();
            itfTblAssignFkmxDownloadService.updateBatchById(ups);
            ups.clear();
        }
    }

    private void getoroldata(Brand brand, OrderBase order, List<OrderRepairOperatingLog> orols, ItfTblAssignFkmxDownload fkmx) {
        OrderRepairOperatingLog add = new OrderRepairOperatingLog();
        add.setWebsitName(order.getWebsitName());
        add.setWebsitId(order.getWebsitId());
        add.setOrderBaseId(order.getId());
        add.setCustomerName(order.getUserName());
        add.setCustomerPhone(order.getUserMobile());
        add.setType(fkmx.getFklb());
        add.setContent(fkmx.getFknr());
        add.setFeedback(fkmx.getFkjg());
        add.setOperator(fkmx.getFkren());
        add.setOperatorNumber(fkmx.getFkrenmc());
        WebsitSH websitSH = publicService.WebsitToSH(fkmx.getFkwdno(), brand.getBrandId());
        add.setOperateWebsit(websitSH.getWdmc());
        add.setWebsitPhone(websitSH.getWdlxdh());
        add.setWorkerOrderStatus(order.getOrderStatus());// 反馈时的工单状态
        add.setBeforeOrderStatus(order.getLastOrderStatus());// 操作前工单状态 - 上一流程状态
        add.setBeforeDispatchStatus(order.getDispatchStatus());// 操作前派工状态
        add.setOrderStatus("DJS"); // 操作后工单状态
        // 总部下来的工单统一为待接收状态-派工状态
        add.setDispatchStatus("DJS"); // 操作后派工状态
        // 附件地址 todo
        add.setPgguid(order.getPgguid());
        add.setFkmxguid(fkmx.getFkmxguid());
        add.setFkid(String.valueOf(fkmx.getFkid()));
        add.setPgid(fkmx.getPgid());
        add.setBrandFileId(fkmx.getScid());
        add.setBrandFileName(fkmx.getScwj());
        add.setWjid(fkmx.getWjid());
        add.setReqSourceNum(fkmx.getQqlyxh());
        add.setUploadNotifyFlag(0);
        add.setCreateTime(fkmx.getFksj());
        add.setUpdateTime(fkmx.getLastModifiedDate());
        add.setCreateBy(fkmx.getCreatedBy());
        add.setUpdateBy(fkmx.getLastModifiedBy());
        orols.add(add);
    }

    private void getOrderInstall(ItfTblAssignDownload item, OrderBase order, Map<String, List<SysDictRef>> sysDictRefDLMapXq,
                                 Map<String, List<SysDictRef>> sysDictRefXLMapXq) {
        OrderRepair or = new OrderRepair();
        or.setOrderBaseId(order.getId());
        or.setCxyzm(item.getCxyzm());
        or.setSffswx(item.getSffswx());
        or.setChaoshiqe(item.getChaoshiqe());
        or.setZbby(item.getZbby());
        or.setZptype(item.getZptype());
        or.setYhif(item.getYhif());
        or.setTsdengji(item.getTsdengji());
        // 需求大类
        or.setDemandTypeId(sysDictRefXLMapXq.get(String.valueOf(item.getSpid())).get(0).getDictCode());
        or.setDemandTypeName(sysDictRefXLMapXq.get(String.valueOf(item.getSpid())).get(0).getDictValue());

        // 需求小类
        if (!item.getXqxiaolei().equals(0L)){
            or.setDemandSmallTypeId(sysDictRefXLMapXq.get(String.valueOf(item.getXqxiaolei())).get(0).getDictCode());
            or.setDemandSmallTypeName(sysDictRefXLMapXq.get(String.valueOf(item.getXqxiaolei())).get(0).getDictValue());
        } else {
            or.setDemandSmallTypeId("");
            or.setDemandSmallTypeName("");
        }
        or.setExtendField1(item.getExtjson1());
        or.setExtendField2(item.getExtjson2());
        or.setExtendField3(item.getExtjson3());
        or.setExtendField4(item.getExtjson4());
        or.setExtendField5(item.getExtjson5());
        or.setCreateBy(order.getCreateBy());
        or.setCreateTime(order.getCreateTime());
        or.setUpdateBy(order.getUpdateBy());
        or.setUpdateTime(order.getUpdateTime());
        orderRepairService.saveOrUpdate(or);
    }

    private void generateMx(Brand brand, ItfTblAssignDownload item, OrderBase order,
                            Map<String, List<SysDictRef>> sysDictRefDLMap,
                            Map<String, List<SysDictRef>> sysDictRefXLMap,
                            Map<String, List<SysDictRef>> sysDictRefXiLieMap) {
        List<ItfTblAssignMxDownload> mxRec = new ArrayList<>();
        List<OrderProduct> addmx = new ArrayList<>();
        List<ItfTblAssignMxDownload> ups = new ArrayList<>();


        // 提取明细表数据
        mxRec = itfTblAssignMxDownloadService.lambdaQuery()
                .eq(ItfTblAssignMxDownload::getDownloadTaskNo, item.getDownloadTaskNo())
                .eq(ItfTblAssignMxDownload::getSynStatus,0)
                .orderByAsc(ItfTblAssignMxDownload::getSynStatus)
                .list();
        for (ItfTblAssignMxDownload rec:mxRec){
            // 提取维修工单明细数据对象
            OrderPar mxreq = new OrderPar();
            mxreq.setSynReqId(Long.valueOf(rec.getSynReqId()));
            mxreq.setBrandId(brand.getBrandId());
            OrderProduct op = repairMxDownloadMapper.getMx(mxreq);

            // 其他部分参数赋值
            op.setOrderBaseId(order.getId());
            op.setPgguid(item.getPgguid());
            op.setMainId(order.getMainId());
            op.setMainName(order.getMainName());
            op.setMainId(sysDictRefDLMap.get(String.valueOf(rec.getSpid())).get(0).getDictCode());
            op.setMainName(sysDictRefDLMap.get(String.valueOf(rec.getSpid())).get(0).getDictValue());
            if (!rec.getXlid().equals(0L)){
                op.setSmallId(sysDictRefXLMap.get(String.valueOf(rec.getXlid())).get(0).getDictCode());
                op.setSmallName(sysDictRefXLMap.get(String.valueOf(rec.getXlid())).get(0).getDictValue());
            } else {
                op.setSmallId(String.valueOf(rec.getXlid()));
                op.setSmallName(rec.getXlmc());
            }

            if (!rec.getXiid().equals("0") && sysDictRefXiLieMap.containsKey(rec.getXiid())){
                op.setSeriesId(sysDictRefXiLieMap.get(rec.getXiid()).get(0).getDictCode());
                op.setSeriesName(sysDictRefXiLieMap.get(rec.getXiid()).get(0).getDictValue());
            } else {
                op.setSeriesId(rec.getXiid());
                op.setSeriesName(rec.getXimc());
            }
            addmx.add(op);

            ItfTblAssignMxDownload up = new ItfTblAssignMxDownload();
            up.setSynReqId(rec.getSynReqId());
            up.setSynStatus(1);
            up.setSynReqTime(new Date());
            ups.add(up);
        }

        if (addmx.size()>0){
            orderProductService.saveBatch(addmx);
            addmx.clear();
            itfTblAssignMxDownloadService.updateBatchById(ups);
            ups.clear();
        }
    }

    private void generateMxUpdate(Brand brand, ItfTblAssignDownload item, OrderBase order,
                            Map<String, List<SysDictRef>> sysDictRefDLMap,
                            Map<String, List<SysDictRef>> sysDictRefXLMap,
                            Map<String, List<SysDictRef>> sysDictRefXiLieMap) {
        List<ItfTblAssignMxDownload> mxRec = new ArrayList<>();
        List<OrderProduct> addmx = new ArrayList<>();
        List<OrderProduct> upmx = new ArrayList<>();
        List<ItfTblAssignMxDownload> ups = new ArrayList<>();
        Map<String, List<OrderProduct>> opsMap = new HashMap<>();

        // 提取现有的数据
        List<OrderProduct> ops = orderProductService.lambdaQuery()
                .eq(OrderProduct::getOrderBaseId,order.getId()).list();

        opsMap = ops.stream().collect(Collectors.groupingBy(OrderProduct::getPgmxid));

        // 提取明细表数据
        mxRec = itfTblAssignMxDownloadService.lambdaQuery()
                .eq(ItfTblAssignMxDownload::getDownloadTaskNo, item.getDownloadTaskNo())
                .eq(ItfTblAssignMxDownload::getSynStatus,0)
                .orderByAsc(ItfTblAssignMxDownload::getSynStatus)
                .list();

        for (ItfTblAssignMxDownload rec:mxRec){
            // 提取维修工单明细数据对象
            OrderPar mxreq = new OrderPar();
            mxreq.setSynReqId(Long.valueOf(rec.getSynReqId()));
            mxreq.setBrandId(brand.getBrandId());
            OrderProduct op = getOrderProduct(item, order, sysDictRefDLMap, sysDictRefXLMap, sysDictRefXiLieMap, rec, mxreq);

            // 存在修改
            if (opsMap.containsKey(op.getPgmxid())){
                getupop(upmx, opsMap, op);
            } else {
                addmx.add(op);
            }

            ItfTblAssignMxDownload up = new ItfTblAssignMxDownload();
            up.setSynReqId(rec.getSynReqId());
            up.setSynStatus(1);
            up.setSynReqTime(new Date());
            ups.add(up);
        }

        if (addmx.size()>0){
            orderProductService.saveBatch(addmx);
            addmx.clear();
        }
        if (upmx.size()>0){
            orderProductService.updateBatchById(upmx);
            addmx.clear();
        }
        if (ups.size()>0){
            itfTblAssignMxDownloadService.updateBatchById(ups);
            ups.clear();
        }

    }

    private void getupop(List<OrderProduct> upmx, Map<String, List<OrderProduct>> opsMap, OrderProduct op) {
        OrderProduct up = new OrderProduct();
        up.setId(opsMap.get(op.getPgmxid()).get(0).getId());
        up.setMainId(op.getMainId());
        up.setMainName(op.getMainName());
        up.setSmallId(op.getSmallId());
        up.setSmallName(op.getSmallName());
        up.setSeriesId(op.getSeriesId());
        up.setSeriesName(op.getSeriesName());
        up.setProductId(op.getProductId());
        up.setProductName(op.getProductName());
        up.setUrgency(op.getUrgency());
        up.setSaleWebsit(op.getSaleWebsit());
        up.setServiceWebsit(op.getServiceWebsit());
        up.setSaleWebsitMobile(op.getSaleWebsitMobile());
        up.setServiceWebsitMobile(op.getServiceWebsitMobile());
        up.setBuyTime(op.getBuyTime());
        upmx.add(up);
    }

    @NotNull
    private OrderProduct getOrderProduct(ItfTblAssignDownload item, OrderBase order, Map<String, List<SysDictRef>> sysDictRefDLMap, Map<String, List<SysDictRef>> sysDictRefXLMap, Map<String, List<SysDictRef>> sysDictRefXiLieMap, ItfTblAssignMxDownload rec, OrderPar mxreq) {
        OrderProduct op = repairMxDownloadMapper.getMx(mxreq);

        // 其他部分参数赋值
        op.setOrderBaseId(order.getId());
        op.setPgguid(item.getPgguid());
        op.setMainId(sysDictRefDLMap.get(String.valueOf(rec.getSpid())).get(0).getDictCode());
        op.setMainName(sysDictRefDLMap.get(String.valueOf(rec.getSpid())).get(0).getDictValue());
        if (!rec.getXlid().equals(0L)){
            op.setSmallId(sysDictRefXLMap.get(String.valueOf(rec.getXlid())).get(0).getDictCode());
            op.setSmallName(sysDictRefXLMap.get(String.valueOf(rec.getXlid())).get(0).getDictValue());
        } else {
            op.setSmallId(String.valueOf(rec.getXlid()));
            op.setSmallName(rec.getXlmc());
        }

        if (!rec.getXiid().equals("0") && sysDictRefXiLieMap.containsKey(rec.getXiid())){
            op.setSeriesId(sysDictRefXiLieMap.get(rec.getXiid()).get(0).getDictCode());
            op.setSeriesName(sysDictRefXiLieMap.get(rec.getXiid()).get(0).getDictValue());
        } else {
            op.setSeriesId(rec.getXiid());
            op.setSeriesName(rec.getXimc());
        }
        return op;
    }

    private OrderBase generateOrder(Brand brand, ItfTblAssignDownload item, List<ItfTblAssignXzydDownload> yds) {
        Boolean isChildInfo = Boolean.FALSE;

        if (yds !=null && yds.size()>0){
            isChildInfo = Boolean.TRUE;
        }

        OrderBase order = saveOrderBase(brand, item, isChildInfo);

        // 总部维修单为待审批改派时 若本地表无改派数据则生成改派数据
        if (order.getOrderStatus().equals("B5")){
            generateDispatch(order);
        }
        // 如果是家用的工单，且工单状态是待中心派工或待网点派工时，写入派嘉讯茂  不做

        // 修改同步状态
        upItfTblAssignDownloadStatus(item, order);

        return order;
    }

    private OrderBase saveOrderBase(Brand brand, ItfTblAssignDownload item, Boolean isChildInfo) {
        // 获取维修工单主表数据
        OrderPar orderPar = new OrderPar();
        orderPar.setSynReqId(item.getSynReqId());
        orderPar.setBrandId(brand.getBrandId());
        OrderBase order = repairDownloadMapper.getOrderBase(orderPar);
        if (isChildInfo){
            order.setIsChildInfo("YES");
        }
        // 部分属性值重置
        orderService.setParame(brand, order);
        // 新增
        orderBaseService.save(order);
        return order;
    }

    public void upItfTblAssignDownloadStatus(ItfTblAssignDownload item, OrderBase order){
        upItfTblAssignDownload(item.getSynReqId(),1,"成功");

        OrderPar prepar = new OrderPar();
        prepar.setSynReqId(item.getSynReqId());
        prepar.setPgid(order.getPgid());
        assignDownloadMapper.updateStatus(prepar);
    }

    private void upItfTblAssignDownload(Long synReqId, Integer synStatus, String synErrMsg) {
        ItfTblAssignDownload up = new ItfTblAssignDownload();
        up.setSynReqId(synReqId);
        up.setSynStatus(synStatus);
        up.setSynTime(new Date());
        up.setSynErrMsg(synErrMsg);
        itfTblAssignDownloadService.updateById(up);
    }

    private void generateDispatch(OrderBase order) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

        List<String> parame = new ArrayList<>();
        parame.add(order.getWebsitId());
        parame.add(order.getReceWebsitId());
        // 改派表中查看是否有待审的数据
        List<OrderApplyDispatch> rec = orderApplyDispatchService.lambdaQuery()
                .eq(OrderApplyDispatch::getOrderBaseId, order.getId())
                .eq(OrderApplyDispatch::getExamineStatus,"WAIT")
                .in(OrderApplyDispatch::getWebsitId,parame).list();
        if (rec.size()<1){
            // 生成改派数据 从维修工单表中读取数据
            OrderApplyDispatch addn = new OrderApplyDispatch();
            addn.setOrderBaseId(order.getId());
            addn.setPgid(order.getPgid());
            addn.setType("REPAIR");
            addn.setWebsitId(order.getWebsitId());
            addn.setWebsitName(order.getWebsitName());
            addn.setApplyContent("申请改派");
            addn.setExamineStatus("WAIT");
            addn.setUserName(order.getUserName());
            addn.setUserMobile(order.getUserMobile());
            addn.setReserveTime(order.getAppointmentTime());
            addn.setReservePeriod(sdf.format(order.getAppointmentTime()));
            addn.setMainId(Integer.valueOf(order.getMainId()));
            addn.setMainName(order.getMainName());
            addn.setCreateTime(order.getCreateTime());
            addn.setUpdateTime(order.getUpdateTime());
            orderApplyDispatchService.save(addn);
        }
    }


}
