package com.zfire.jiasm.syncdata.upload.order;

import com.zfire.jiasm.syncdata.constant.GDAppointStatusEnum;
import com.zfire.jiasm.syncdata.constant.GDOrderStatusEnum;
import com.zfire.jiasm.syncdata.constant.OrderTypeEnum;
import com.zfire.jiasm.syncdata.plus.entity.*;
import com.zfire.jiasm.syncdata.plus.service.*;
import com.zfire.jiasm.syncdata.service.PrepareService;
import com.zfire.jiasm.syncdata.service.PublicService;
import com.zfire.jiasm.syncdata.utils.PrepareTaskLog;
import lombok.extern.slf4j.Slf4j;
import org.jetbrains.annotations.NotNull;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 生成上传任务清单 out_jiasm_to_gree_upload_task_list
 * 1:从需要上传的订单数据表中提取数据 out_jiasm_to_gree_uploadnotifylist
 *:2:对应表中提取表中数据 生成上传任务清单 out_jiasm_to_gree_upload_task_list
 */
@Slf4j
@Component
public class RepairProduceUploadList {
    @Resource
    RepairProduceUploadList repairProduceUploadList;
    @Resource
    PrepareService prepareService;
    @Resource
    OutJiasmToGreeUploadTaskListService outJiasmToGreeUploadTaskListService;
    @Resource
    OrderBaseService orderBaseService;
    @Resource
    OutJiasmToGreeUploadnotifylistService outJiasmToGreeUploadnotifylistService;
    @Resource
    PublicService publicService;
    @Resource
    OrderProductService orderProductService;
    @Resource
    OrderRepairOperatingLogService orderRepairOperatingLogService;


    @Scheduled(fixedDelay = 1 * 5 * 1000)
    private void produceList(){
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        List<OutJiasmToGreeUploadTaskList> ojtgutls = new ArrayList<>();
        if (1==1){
            return;
        }

          log.info("开始预处理需要上传的维修工单数据: \uF0B7{}",sdf.format(new Date()));

        // 从需要上传的订单数据表中提取数据
        List<OutJiasmToGreeUploadnotifylist> taskResults = outJiasmToGreeUploadnotifylistService.lambdaQuery()
                .eq(OutJiasmToGreeUploadnotifylist::getSynStatus,0)
                .orderByAsc(OutJiasmToGreeUploadnotifylist::getSynReqId).list();
        try{
            for (OutJiasmToGreeUploadnotifylist item : taskResults) {
                Long itemSynReqId = item.getSynReqId();
                // 同步的表名
                String itemTableName = item.getTableName();
                // 同步表中的工单号
                String itemOrderNo = item.getOrderNo();
                // 同步表中的主键id
                String itemTableNameId = item.getId();
                // 工单类型
                String itemOrderType = item.getOrderType();

                // 数据同步预处理规则表中提取对应表名的规则
                Map<String, Object> row = prepareService.getTaskInfo(itemTableName);
                int taskId = (Integer) row.get("id");
                String srcTabName = (String) row.get("src_tab_name");

                String errorMessage = null;
                try{
                    // 维修工单主表处理 66、661
                    if (itemTableName.equals("order_base") &&
                            itemOrderType.equals(OrderTypeEnum.ORDER_TYPE_REPAIR.getCode())){
                        // 过程名，源表中的ID，源表中的数据最后修改时间，源表中的工单号，源表中的id,空值，false
                        // TL_PrePare_task_by_fa_order_repair
                        repairProduceUploadList.orderBaseProess(itemTableName, itemOrderNo, itemTableNameId, ojtgutls, item.getSynReqId());
                    }
                    // 维修工单产品信息 68
                    if (itemTableName.equals("order_product") &&
                            itemOrderType.equals(OrderTypeEnum.ORDER_TYPE_REPAIR.getCode())){
                        // 过程名，源表中的ID，源表中的数据最后修改时间，源表中的工单号，源表中的id,空值，false
                        // TL_PrePare_task_by_fa_order_repair_product
                        repairProduceUploadList.orderProductProess(item, itemTableName, itemOrderNo, itemTableNameId);
                    }
                    // 维修工单操作明细 67
                    if (itemTableName.equals("order_repair_operating_log")){
                         // TL_PrePare_task_by_fa_order_repair_operating_log
                        repairProduceUploadList.logProess(item, itemTableName, itemOrderNo, itemTableNameId);
                    }



                } catch (Exception ex) {
                    errorMessage = ex.getMessage();
                    updateError(errorMessage, item);
                } finally {
                    PrepareTaskLog.writeMessage(genInfoData(taskId, srcTabName, false, errorMessage));
                }
            }
            log.info("预处理需要上传的维修工单数据完成: \uF0B7{}",sdf.format(new Date()));

        } catch (Exception ex) {
        ex.printStackTrace();
        log.info("开始预处理需要上传的维修工单数据出错");
    }


    }

    @Transactional(rollbackFor = Exception.class)
    public void logProess(OutJiasmToGreeUploadnotifylist item, String itemTableName, String itemOrderNo, String itemTableNameId) {
        // 提取日志数据
        OrderRepairOperatingLog org = orderRepairOperatingLogService.lambdaQuery()
                .eq(OrderRepairOperatingLog::getId, itemTableNameId).one();
        if (org !=null){
            // 生成67类型上传任务数据
            // 单号，主键，同步任务生成时间，同步的表名，工单类型，任务类型
            OutJiasmToGreeUploadTaskList add = publicService.getUploadTaskList(itemOrderNo, itemTableNameId,org.getUpdateTime(), itemTableName,
                    OrderTypeEnum.ORDER_TYPE_REPAIR.getCode(),67);
            outJiasmToGreeUploadTaskListService.save(add);

            // 需要上传的订单数据表中处理状态同步为已处理
            OutJiasmToGreeUploadnotifylist upn = getOutJiasmToGreeUploadnotifylist(item.getSynReqId());
            outJiasmToGreeUploadnotifylistService.updateById(upn);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public void orderProductProess(OutJiasmToGreeUploadnotifylist item, String itemTableName, String itemOrderNo, String itemTableNameId) {
        // 提取工单机型
        OrderProduct op = orderProductService.lambdaQuery()
                .eq(OrderProduct::getId, itemTableNameId).one();
        if (op!=null){
            // 生成68类型上传任务数据
            // 单号，主键，同步任务生成时间，同步的表名，工单类型，任务类型
            OutJiasmToGreeUploadTaskList add = publicService.getUploadTaskList(itemOrderNo, itemTableNameId,op.getUpdateTime(), itemTableName,
                    OrderTypeEnum.ORDER_TYPE_REPAIR.getCode(),68);
            outJiasmToGreeUploadTaskListService.save(add);

            // 需要上传的订单数据表中处理状态同步为已处理
            OutJiasmToGreeUploadnotifylist upn = getOutJiasmToGreeUploadnotifylist(item.getSynReqId());
            outJiasmToGreeUploadnotifylistService.updateById(upn);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public void orderBaseProess(String itemTableName, String itemOrderNo, String itemTableNameId,
                                 List<OutJiasmToGreeUploadTaskList> ojtgutls, Long synReqId) {
        // 提取工单数据
        OrderBase orderBase = orderBaseService.lambdaQuery()
                .eq(OrderBase::getId, itemTableNameId).one();
        // 生成任务清单数据
        if (orderBase != null){
            // 删除工单更新被忽略

            // 生成66类型上传任务数据
            // 单号，主键，同步任务生成时间，同步的表名，工单类型，任务类型
            OutJiasmToGreeUploadTaskList add = publicService.getUploadTaskList(itemOrderNo, itemTableNameId,orderBase.getUpdateTime(), itemTableName,
                    orderBase.getOrderType(),66);

            ojtgutls.add(add);

            // 生成661类型上传任务数据
            OrderBase up = generateTaskList(itemTableName, itemOrderNo, itemTableNameId, orderBase, ojtgutls);


            if (ojtgutls.size()>0){
                outJiasmToGreeUploadTaskListService.saveBatch(ojtgutls);
                ojtgutls.clear();
            }
            if (up !=null ){
                orderBaseService.updateById(up);
                orderBase.setOutCjDownloadFlag(1);
            }

            // 需要上传的订单数据表中处理状态同步为已处理
            OutJiasmToGreeUploadnotifylist upn = getOutJiasmToGreeUploadnotifylist(synReqId);
            outJiasmToGreeUploadnotifylistService.updateById(upn);

        }
    }

    @NotNull
    private OutJiasmToGreeUploadnotifylist getOutJiasmToGreeUploadnotifylist(Long synReqId) {
        OutJiasmToGreeUploadnotifylist up = new OutJiasmToGreeUploadnotifylist();
        up.setSynReqId(synReqId);
        up.setSynStatus(1);
        up.setSynReqTime(new Date());
        return up;
    }

    private OrderBase generateTaskList(String itemTableName, String itemOrderNo, String itemTableNameId, OrderBase orderBase, List<OutJiasmToGreeUploadTaskList> ojtgutls) {
        // 工单状态(8, 9, 10, 11, 12, 13, 14) 派工状态:(13, 14, 15) 未下载总部采集数据
        if ((orderBase.getOrderStatus().equals(GDOrderStatusEnum.REPAIR_STATUS_G8.getCode()) ||
                orderBase.getOrderStatus().equals(GDOrderStatusEnum.REPAIR_STATUS_G9.getCode()) ||
                orderBase.getOrderStatus().equals(GDOrderStatusEnum.REPAIR_STATUS_G10.getCode()) ||
                orderBase.getOrderStatus().equals(GDOrderStatusEnum.REPAIR_STATUS_G11.getCode()) ||
                orderBase.getOrderStatus().equals(GDOrderStatusEnum.REPAIR_STATUS_G12.getCode())||
                orderBase.getOrderStatus().equals(GDOrderStatusEnum.REPAIR_STATUS_G13.getCode())||
                orderBase.getOrderStatus().equals(GDOrderStatusEnum.REPAIR_STATUS_G14.getCode())) &&
                (orderBase.getDispatchStatus().equals(GDAppointStatusEnum.REPAIR_STATUS_G14_0) ||
                        orderBase.getDispatchStatus().equals(GDAppointStatusEnum.REPAIR_STATUS_G14_1)||
                        orderBase.getDispatchStatus().equals(GDAppointStatusEnum.REPAIR_STATUS_G15_0)||
                        orderBase.getDispatchStatus().equals(GDAppointStatusEnum.REPAIR_STATUS_G15_1)||
                        orderBase.getDispatchStatus().equals(GDAppointStatusEnum.REPAIR_STATUS_G15_2)) &&
                orderBase.getOutCjDownloadFlag().equals(0)){
            OutJiasmToGreeUploadTaskList add = publicService.getUploadTaskList(itemOrderNo, itemTableNameId, itemTableName,
                    orderBase.getOrderType(),661);
            ojtgutls.add(add);

            OrderBase up = new OrderBase();
            up.setId(orderBase.getId());
            up.setOutCjDownloadFlag(1);
            return up;
        }
        return null;
    }

    public void updateError(String errMessage, OutJiasmToGreeUploadnotifylist item) {
        if (errMessage != null && errMessage.trim().length() > 255) {
            errMessage = errMessage.substring(0, 255);
        }
        OutJiasmToGreeUploadnotifylist up = new OutJiasmToGreeUploadnotifylist();
        up.setSynReqId(item.getSynReqId());
        up.setSynTimes(item.getSynTimes()+1);
        up.setSynErrMsg(errMessage);
        up.setSynReqTime(new Date());
        outJiasmToGreeUploadnotifylistService.updateById(up);
    }

    private String genInfoData(int taskId, String srcTabName, boolean isTimeOut, String errorMessage) {
        String result = "预处理任务[" + taskId + "," + srcTabName + "] " + (isTimeOut ? "超时!!!!" : "正常") + " "
                + (errorMessage != null ? "错误:[" + errorMessage + "]" : "");
        return result;
    }





}
